/* ------------------------------------------------------------------------- */
/*
 *  table.h
 *
 *  Copyright (c) 2004 - 2008, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Thu 30 Aug 2007 11:32:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TABLE_H
#define CLX_TABLE_H

#include <string>
#include <vector>
#include <istream>
#include <fstream>
#include <stdexcept>
#include "tokenizer.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  table
	/* --------------------------------------------------------------------- */
	template <
		class TokenizerFunc,
		class Type = std::string,
		template <class> class Container = std::vector
	>
	class table : public Container<Container<Type> > {
	public:
		typedef Type value_type;
		typedef TokenizerFunc functor;
		typedef Container<Container<Type> > container;
		typedef basic_tokenizer<TokenizerFunc, Type> parser;
		typedef typename parser::char_type char_type;
		typedef typename parser::string_type string_type;
		typedef std::basic_istream<char_type> istream;
		
		// constructor and destructor
		explicit table(const functor& f = functor()) :
			container(), parser_(f) { this->reset(); }
		
		explicit table(istream& sin, const functor& f = functor()) :
			container(), parser_(f) {
			this->reset();
			this->read(sin);
		}
		
		explicit table(const char_type* path, const functor& f = functor()) :
			container(), parser_(f) {
			this->reset();
			this->read(path);
		}
		
		explicit table(const string_type& path, const functor& f = functor()) :
			container(), parser_(f) {
			this->reset();
			this->read(path);
		}
		
		virtual ~table() {}
		
		// operator
		virtual void reset() {
			this->clear();
			parser_.reset();
		}
		
		virtual bool readline(istream& sin) {
			string_type tmp;
			if (!std::getline(sin, tmp)) return false;
			this->insert(this->end(), parser_(tmp));
			return true;
		}
		
		table& read(istream& sin) {
			while (this->readline(sin));
			return *this;
		}
		
		table& read(const char_type* path) {
			std::basic_ifstream<char_type> fp(path);
			if (!fp.is_open()) throw std::runtime_error("No such file or directory");
			this->read(fp);
			fp.close();
			return *this;
		}
		
		table& read(const string_type& path) {
			return this->read(path.c_str());
		}
		
	private:
		parser parser_;
	};
}

#endif // CLX_TABLE_H
