/* ------------------------------------------------------------------------- */
/*
 *  winsock_util.h
 *
 *  Copyright (c) 2004 - 2008, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Fri 08 Feb 2008 01:23:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_WINSOCK_UTIL_H
#define CLX_WINSOCK_UTIL_H

#include <winsock2.h>
#pragma comment(lib, "ws2_32.lib")

/* ------------------------------------------------------------------------- */
//  sleep
/* ------------------------------------------------------------------------- */
void sleep(int sec) {
	::Sleep(sec);
}

/* ------------------------------------------------------------------------- */
/*
 *  inet_aton
 *
 *  WinSock does not have the inet_aton() function.
 */
/* ------------------------------------------------------------------------- */
int inet_aton(const char* s, struct in_addr* addr) {
	int tmp = ::inet_addr(s);
	if (tmp != INADDR_NONE || ::strcmp(s, "255.255.255.255") == 0) {
		if (addr) addr->s_addr = tmp;
		return 1;
	}
	return 0;
}

namespace clx {
	/* --------------------------------------------------------------------- */
	//  winsock_init
	/* --------------------------------------------------------------------- */
	class winsock_init {
	public:
		winsock_init() {
			WSADATA dat;
			::WSAStartup(MAKEWORD(2, 0), &dat);
		}
		
		~winsock_init() {
			::WSACleanup();
		}
	private:
		winsock_init(const winsock_init& cp);
		winsock_init& operator=(const winsock_init& cp);
	};
	
	static winsock_init ws_init_;
}

#endif // CLX_WINSOCK_UTIL_H
